#-----------------------------------------------------------------------------
# Name:        Petition.py
#
# Author:      Song Huang
#
# Created:     2004/09/08
# Copyright:   (c) 2004 ossacc.org
# License:     GPL
#-----------------------------------------------------------------------------

import os.path

from time import time
from App.Common import package_home
from OFS.SimpleItem import SimpleItem
from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CMFCorePermissions import ManagePortal
from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate
from interfaces.portal_subsitepetition import IPetition

def delObjects(cont,ids):
    delids=[id for id in ids if hasattr(cont,id)]
    cont.manage_delObjects(delids)

class Petition(SimpleItem):
    ''' class storing information about an petition'''

    __implements__ = IPetition

    meta_type="Sub Site Petition"
    manage_options=(
        {'label':'view','action':'manage_workspace'},
        )
    manage_workspace=ZopePageTemplate('manage_workspace',open(os.path.join(package_home(globals()),'www','petition_overview.pt')).read())
    security = ClassSecurityInfo()

    def __init__(self, newid="", name="", company="", worktitle="", zip="", address="",
                 telno="", e_mail="", description="", domain_name="", base_name="", screenshot=""):
        self.id=newid
        self.name=name
        self.company=company
        self.title="%s-%s" % (company, name)
        self.worktitle=worktitle
        self.zip=zip
        self.address=address
        self.telno=telno
        self.e_mail=e_mail
        self.description=description
        self.domain_name=domain_name
        self.base_name=base_name
        self.screenshot=screenshot
        # status == 0:waiting-confirm 1:pending, 2:rejected, 3:allowed
        self.status=0
        # modified: {"user": uid, "timestamp": time, "msg": logmsg}
        self.modified=[]
        self.storageId = ''

    security.declareProtected(ManagePortal, 'update')
    def update(self,name="",company="",worktitle="",zip="",address="",
                 telno="",e_mail="",description="",domain_name="",screenshot=""):
        ''' updates the petition attributes '''
        error = []
        logmsg = ""
        if self.status != 3:
            if domain_name != "" and self.domain_name != domain_name:
                logmsg += "The domain name change from " + self.domain_name + " to " + domain_name + ".\n"
                self.domain_name = domain_name
            if screenshot != "" and self.screenshot != screenshot:
                logmsg += "The screenshot change from " + self.screenshot + " to " + screenshot + ".\n"
                self.screenshot = screenshot
        else:
            if domain_name != "" or domain_name != self.domain_name:
                error.append('domain_name')
            if screenshot != "" or screenshot != self.screenshot:
                error.append('screenshot')
            if error:
                self.REQUEST.set('error',error)
        if name != "" and name != self.name:
            logmsg += "The name change from " + self.name + " to " + name + ".\n"
            self.name = name
        if company != "" and company != self.company:
            logmsg += "The company change from " + self.company + " to " + company + ".\n"
            self.company = company
        if worktitle != "" and worktitle != self.worktitle:
            logmsg += "The title change from " + self.worktitle + " to " + worktitle + ".\n"
            self.worktitle = worktitle
        if zip != "" and zip != self.zip:
            logmsg += "The zip change from " + self.zip + " to " + zip + ".\n"
            self.zip = zip
        if address != "" and address != self.address:
            logmsg += "The address change from " + self.address + " to " + address + ".\n"
            self.address = address
        if telno != "" and telno != self.telno:
            logmsg += "The telno change from " + self.telno + " to " + telno + ".\n"
            self.telno = telno
        if e_mail != "" and e_mail != self.e_mail:
            logmsg += "The e_mail change from " + self.e_mail + " to " + e_mail + ".\n"
            self.e_mail = e_mail
        if description != "" and description != self.description:
            logmsg += "The description change from " + self.description + " to " + description + ".\n"
            self.description = description
        if logmsg:
            self.log(logmsg)
        return self.REQUEST.RESPONSE.redirect(self.REQUEST['HTTP_REFERER'])

    security.declarePrivate('log')
    def log(self, logmsg):
        ''' adds a log to the transcript '''
        mtool = getToolByName(self, 'portal_membership')
        memberid = mtool.getAuthenticatedMember().getId()
        self.modified.insert(0,{'user': memberid, 'timestamp':time(), 'msg':logmsg})

    def isWaitConfirm(self,days=10):
        ''' determines if the petition is wait to confirm '''
        deadline = 60*60*24*days # default 10 days
        if self.status == 0 and (time() - self.modified[-1]['timestamp']) < deadline:
            return 1
        else: return 0

    def isPending(self):
        ''' determines if the petition is pending '''
        if self.status == 1:
            return 1
        else: return 0

    def isRejected(self):
        ''' determines if the petition is rejected '''
        if self.status == 2:
            return 1
        else: return 0

    def isAllowed(self):
        ''' determines if the sub site was running '''
        if self.status == 3:
            return 1
        else: return 0

    security.declareProtected(ManagePortal, 'setPending')
    def setPending(self, formCheck=None):
        ''' set the petition's status to pending '''
        self.status = 1
        comment = self.REQUEST.get('comment', None)
        if comment:
            self.log(comment + '(Change status to pending.)')
        else:
            self.log('(Change status to pending.)')
        if formCheck:
            return self.REQUEST.RESPONSE.redirect(self.REQUEST['HTTP_REFERER'])

    security.declareProtected(ManagePortal, 'setReject')
    def setReject(self, formCheck=None):
        ''' set the petition's status to rejected '''
        self.status = 2
        comment = self.REQUEST.get('comment', None)
        if comment:
            self.log(comment + '(Change status to reject.)')
        else:
            self.log('(Change status to reject.)')
        if formCheck:
            return self.REQUEST.RESPONSE.redirect(self.REQUEST['HTTP_REFERER'])

    security.declareProtected(ManagePortal, 'setAllow')
    def setAllow(self, formCheck=None):
        ''' set the petition's status to allowed '''
        self.status = 3
        comment = self.REQUEST.get('comment', None)
        if comment:
            self.log(comment + '(Change status to allow.)')
        else:
            self.log('(Change status to allow.)')
        if formCheck:
            return self.REQUEST.RESPONSE.redirect(self.REQUEST['HTTP_REFERER'])

    security.declareProtected('View', 'Created')
    def Created(self):
        ''' return the petition's creating time '''
        return self.modified[-1]['timestamp']

    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        self.log('Created!')
