#-----------------------------------------------------------------------------
# Name:        Storage.py
#
# Author:      Song Huang
#
# Created:     2004/09/22
# Copyright:   (c) 2004 ossacc.org
# License:     GPL
#-----------------------------------------------------------------------------

import os.path

from time import time
from App.Common import package_home
from OFS.SimpleItem import SimpleItem
from AccessControl import ClassSecurityInfo
from Products.CMFCore.CMFCorePermissions import ManagePortal
from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate
from Products.ZODBMountPoint.MountedObject import manage_addMounts
from Products.CMFCore.utils import getToolByName
from interfaces.portal_subsitepetition import IStorage

class Storage(SimpleItem):
    ''' class storing information about an storage '''

    __implements__ = IStorage

    meta_type="Sub Site Storage"
    manage_options=(
        {'label':'view','action':'manage_workspace'},
        )
    manage_workspace=ZopePageTemplate('manage_workspace',open(os.path.join(package_home(globals()),'www','storage_overview.pt')).read())
    security = ClassSecurityInfo()
    error = 1

    def __init__(self, newid="", petitionId=""):
        ''' constructor '''
        self.id = newid
        self.petitionId = petitionId
        self.path = ''
        self.created = time()
        self.assignPI = None
        self.includepath = ''

    security.declareProtected(ManagePortal, 'isAvailable')
    def isAvailable(self):
        ''' if not regester petitionId '''
        return not self.petitionId

    security.declareProtected(ManagePortal, 'isUsed')
    def isUsed(self):
        ''' if regester petitionId '''
        return not not self.petitionId

    security.declareProtected(ManagePortal, 'regesterPI')
    def regesterPI(self, pi):
        ''' regester petitionId and add mount point '''
        if self.isAvailable():
            self.petitionId = pi
            portal = getToolByName(self, 'portal_url').getPortalObject()
            sspt = getToolByName(self, 'portal_subsitepetition')
            SSFolder = getattr(portal, sspt.subsitefolder)
            self.path = '/'.join(SSFolder.getPhysicalPath()) + '/' + self.id
            manage_addMounts(SSFolder, (self.path,))
            self.assignPI = time()
        else:
            return error
        return None

    security.declareProtected(ManagePortal, 'regesterPI')
    def deletePI(self):
        ''' delete petitionId and delete mount point '''
        if self.isUsed():
            self.petitionId = ''
            portal = getToolByName(self, 'portal_url').getPortalObject()
            sspt = getToolByName(self, 'portal_subsitepetition')
            SSFolder = getattr(portal, sspt.subsitefolder)
            SSFolder.manage_delObjects([self.id,])
            self.assignPI = None
        else:
            return error
        return None

    security.declareProtected(ManagePortal, 'Created')
    def Created(self):
        ''' return the storage's creating time '''
        return self.created
    
    def manage_beforeDelete(self, item, container):
        ''' erase "%%include" line inside config file, and delete storage setting file '''
        zopeconf = os.path.join(INSTANCE_HOME, 'etc', 'zope.conf')
        cmd = "grep -v '^%s$' %s > %s" % (self.includepath, zopeconf, zopeconf+'.new')
        os.system(cmd)
        cmd = "mv -f %s %s" % (zopeconf+'.new', zopeconf)
        os.system(cmd)
        cmd = "rm -f %s" % self.includepath
        os.system(cmd)






