#-----------------------------------------------------------------------------
# Name:        SubSitePetitionTool.py
#
# Author:      Song Huang
#
# Created:     2004/09/07
# Copyright:   (c) 2004 ossacc.org
# License:     GPL
#-----------------------------------------------------------------------------

import os
from string import letters, digits
from time import time, sleep
from random import randint
from base64 import encodestring
from socket import gethostbyname

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from App.Common import package_home
from App.Product import ProductFolder
from OFS.SimpleItem import SimpleItem
from OFS.ObjectManager import ObjectManager
from OFS.PropertyManager import PropertyManager
from OFS.Folder import Folder

from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.CMFCore.CMFCorePermissions import ManagePortal
from Products.CMFCore.DirectoryView import addDirectoryViews
from Products.CMFPlone import custom_policies
from Products.CMFPlone.Portal import manage_addSite
from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate

from Petition import Petition
from Storage import Storage
from config import *
from interfaces.portal_subsitepetition import ISubSitePetitionTool

import zLOG
import Lifetime

error = 1

default_frontpage=r"""
You can customize this frontpage by clicking the edit tab on this document if you
have the correct permissions. Create folders and put content in those folders.
Folders will show up in the navigation box if they are published. It's a very
simple and powerful system.
"""

def addSubSitePetitionTool(self,REQUEST=None):
    ''' add sub site petition tool to plone site '''
    # if inside subsitefolder
    if getattr(self, 'inSubSiteFolder', None):
        return 'here inside of subsitefolder!'
    # add sub site petition tool
    sspt=SubSitePetitionTool()
    self._setObject('portal_subsitepetition',sspt,set_owner=0)
    # make subsite folder
    root = self.getPhysicalRoot()
    objids = root.objectIds()
    if sspt.subsitefolder in objids:
        for i in range(1,20):
            ssf = sspt.subsitefolder + '_%2.2d' % i
            if ssf not in objids:
                break
        sspt.subsitefolder = ssf
    ssf = SubSiteFolder(sspt.subsitefolder)
    ssf.title = 'Sub Site'
    root._setObject(sspt.subsitefolder, ssf)
    # add skin
    skinstool = getToolByName(self, 'portal_skins')
    skin_name = 'subsitepetition'
    if skin_name not in skinstool.objectIds():
        addDirectoryViews(skinstool, 'skins', globals())
    for skinName in skinstool.getSkinSelections():
        path = skinstool.getSkinPath(skinName) 
        path = [i.strip() for i in  path.split(',')]
        if skin_name not in path:
            path.append(skin_name)
        path = ','.join(path)
        skinstool.addSkinSelection(skinName, path)
    # add portlet
    lslots = self.getProperty('left_slots', [])
    portlet = 'here/portlet_subsite/macros/portlet'
    if portlet not in lslots:
        lslots = lslots + (portlet,)
    self.manage_changeProperties({'left_slots': lslots})
    # redirect where from
    if REQUEST:
        return REQUEST.RESPONSE.redirect(REQUEST['HTTP_REFERER'])

class SubSiteFolder( Folder ): 
    meta_type='SubSite Folder'

    _properties = ({'id': 'title',          'type': 'string'},
                   {'id': 'hiddenProducts', 'type': 'lines'})

    title = 'SubSite Folder'
    hiddenProducts = ['SubSitePetitionTool',]

    inSubSiteFolder = 1

    def __init__(self, id=None):
        if id is not None:
            self.id = str(id)

    def getHiddenProducts(self):
        return self.getProperty('hiddenProducts', [])

class SubSitePetitionTool( UniqueObject, ObjectManager, SimpleItem, PropertyManager ):

    __implements__ = ISubSitePetitionTool

    meta_type = 'CMF SubSitePetition Tool'
    id='portal_subsitepetition'

    security = ClassSecurityInfo()

    manage_options=(
        {'label':'Petitions','action':'PetitionList'},
        {'label':'Storages','action':'StorageList'},
        ) + PropertyManager.manage_options + ObjectManager.manage_options 
    
    _properties = ({'id': 'title',       'type': 'string'},
                   {'id': 'base_name',   'type': 'string'},
                   {'id': 'right_slots', 'type': 'lines'})
    title= 'SubSite Petition Tool'
    base_name = ''
    right_slots = []

    PetitionList=ZopePageTemplate('PetitionList',open(os.path.join(package_home(globals()),'www','petition_list.pt')).read())
    StorageList=ZopePageTemplate('StorageList',open(os.path.join(package_home(globals()),'www','storage_list.pt')).read())
    security = ClassSecurityInfo()
    
    def manage_afterAdd(self, item, container):
        self.manage_changeProperties(base_name=self.REQUEST.get_header('SERVER_NAME',''))
    
    def __init__(self):
        self.id = 'portal_subsitepetition'
        self.subsitefolder = 'SubSite'

    security.declarePublic('havePending')
    def havePending(self):
        if self.listPetitions():
            return 1
        return None

    security.declareProtected(ManagePortal, 'listPetitions')
    def listPetitions(self, pendingOnly=1):
        ''' returns a list of petitions 
            -> list of dicts with keys:(id,name,description,timestamp,status)'''
        if pendingOnly:
            pids = [o.id for o in self.objectValues('Sub Site Petition') if o.isPending()]
        else:
            deadline = 60*60*24*30 # 30 days
            pids = [o.id for o in self.objectValues('Sub Site Petition')]

        res=[]

        for r in pids:
            p=self._getOb(r, None)
            if p:
                res.append({'id':          r, 
                            'name':        getattr(p,'name'), 
                            'description': p.description,
                            'timestamp':   p.Created(),
                            'status':      p.status
                            })

        return res

    security.declarePrivate('newPetitionId')
    def newPetitionId(self):
        ''' return a petition's id for createPetiton function ->
            current time for second + random numeral '''
        return '%d%3.3d' % (time(), randint(1,999))

    security.declareProtected('View', 'createPetition')
    def createPetition(self, name="",company="",worktitle="",zip="",address="",
        telno="",e_mail="",description="",domain_name="",base_name="",screenshot="",REQUEST=None):
        ''' create a petition '''
        if REQUEST:
            newid=self.newPetitionId()
            petition=Petition(newid, name, company, worktitle, zip, address, telno,
                              e_mail, description, domain_name, base_name, screenshot)
            self._setObject(newid,petition)
            x=randint(1,9)
            result = "%s%s%s" % ( chr(x+64), newid, x )
            return result

    security.declareProtected('View', 'confirmPetition')
    def confirmPetition(self, checksum):
        ''' prefix:[A-J] that mapping to postfix:[1-9] '''
        checknum = int(checksum[-1])
        if checknum < 1 or checknum > 9:
            return self.REQUEST.RESPONSE.redirect(self.absolute_url()+'/confirmed?err=1')
        checkchar = ord(checksum[0]) - 64
        if checkchar != checknum:
            return self.REQUEST.RESPONSE.redirect(self.absolute_url()+'/confirmed?err=2')
        petition = checksum[1:-1]
        if petition in self.objectIds():
            p=self._getOb(petition, None)
            if p.isWaitConfirm():
                p.setPending()
                return self.REQUEST.RESPONSE.redirect(self.absolute_url()+'/confirmed')
            else:
                return self.REQUEST.RESPONSE.redirect(self.absolute_url()+'/confirmed?err=4')

    security.declareProtected(ManagePortal, 'RejectPetition')
    def RejectPetition(self, id):
        ''' Reject petition by id. '''
        p=self._getOb(id,None)
        if not p: return 0
        if p.isRejected(): return 0
        if p.isAllowed(): return 0
        #------------------------
        # send welcome msg.
        msg = self.REQUEST.get('comment', 'no comment.')
        self.sendto('rejected_template', 
                    {'e_mail':      p.e_mail, 
                     'subject':     'Rejected.', 
                     'domain_name': p.domain_name+'.'+p.base_name,
                     'msg':         msg})
        self.sendto('rejected_template', 
                    {'e_mail':      self.email_from_address, 
                     'subject':     'Rejected.', 
                     'domain_name': p.domain_name+'.'+p.base_name,
                    'msg':          msg})
        p.setReject()
        return 1

    security.declareProtected(ManagePortal, 'RejectPetitions')
    def RejectPetitions(self, petition=[]):
        ''' Reject petition by ids. '''
        results = 0
        for id in petition:
            result = self.RejectPetition(id)
            if result:
                results += 1
        
        if results > 1:
            plural = 's'
        else:
            plural = ''
        return self.REQUEST.RESPONSE.redirect(self.REQUEST['HTTP_REFERER']+'?portal_status_message=Reject %s petition%s' % (results, plural))

    security.declarePublic('validateDN')
    def validateDN(self, domainName, baseName):
        ''' return none if it's unused, if not return error '''
        for c in domainName:
            if c not in letters+digits+'-':
                self.REQUEST.set('err', 'The domain name you selected is not valid. Please choose another.(%s)' % c)
                return error
        try:
            ip = gethostbyname(domainName+'.'+baseName)
        except:
            return None
        else:
            self.REQUEST.set('err', 'The domain name you selected is already in use. Please choose another.')
            return error

    security.declarePrivate('addDN')
    def addDN(self, dn):
        ''' add dn to dns server '''
        cmd = """echo -e "server %s\nupdate add %s %s A %s\nsend\n" | nsupdate """
        chstdin, chstdout, chstderr = os.popen3(cmd % (DDNS_SERVER, dn, DDNS_TTL, HOST_IP))
        errors = chstderr.read()
        if errors:
            self.REQUEST.set('err', errors)
            return error
        return None

    security.declarePrivate('addAPAVH')
    def addAPAVH(self, p):
        ''' add apache virtual host setting '''
        serveradmin = p.e_mail
        domain_name = p.domain_name+'.'+p.base_name

        ssPath = '/%s/%s/%s' % (self.subsitefolder, p.storageId, p.id)

        # write to virtual host config file
        file = open(os.path.join(APACHE_VH_FOLDER, domain_name), 'w')
        file.write(VH_SETTING % (serveradmin, domain_name, domain_name, ssPath))
        file.close()

        # reload apache
        cmd = "sudo %s reload" % APACHE_CTRL
        chstdin, chstdout, chstderr = os.popen3(cmd)
        errors = chstderr.read()
        if errors:
            self.REQUEST.set('err', errors)
            return error
        return None

    security.declarePrivate('addSubSite')
    def addSubSite(self, p):
        ''' add sub site in plone site '''
        SSFolder = getattr(self, self.subsitefolder, None)
        storage = getattr(SSFolder, p.storageId, None)
        if storage:
            # add site
            manage_addSite(storage, p.id, title=p.company, description='',
                           create_userfolder=1,
                           email_from_address=p.e_mail,
                           email_from_name='Portal Administrator',
                           validate_email=0,
                           custom_policy=p.screenshot)
            newportal = getattr(storage, p.id, None)
            if newportal:
                # add manager
                portal_registration = newportal.portal_registration
                site_properties = newportal.portal_properties.site_properties
                username = p.domain_name
                password = portal_registration.generatePassword()
                portal_registration.addMember(username, password, 
                                              roles=('Manager',), 
                                              properties={'username': username,
                                                          'email': p.e_mail}
                                              )
                portal_registration.registeredNotify(username)
                # refine index_html
                idx = getattr(newportal, 'index_html')
                idx.setTitle('Welcome to Plone')
                idx.setDescription('This welcome page is used to introduce you'+\
                                   ' to the Plone Content Management System.')
                idx.setFormat('structured-text')
                if idx.meta_type == 'Document':
                    # CMFDefault document
                    idx.edit('structured-text', default_frontpage)
                else:
                    idx.edit(text=default_frontpage)
                idx.reindexObject()
            return None
        self.REQUEST.set('err', 'No more storage!')
        return error

    security.declarePrivate('createSubSite')
    def createSubSite(self, p):
        ''' batch procedure to create sub site '''
        #------------------------
        # update domain_name.
        if SETUP_DDNS:
            dn = p.domain_name+'.'+p.base_name
            if not self.validateDN(p.domain_name, p.base_name):
                self.addDN(dn)
            else: 
                self.REQUEST.set('msg', 'updata ddns: %s' % self.REQUEST['err'])
                return None
        #------------------------
        # make sure the SubSite folder exist.
        root = self.getPhysicalRoot()
        SSFolder = getattr(root, self.subsitefolder, None)
        if not SSFolder:
            ssf = SubSiteFolder(self.subsitefolder)
            ssf.title = 'Sub Site'
            root._setObject(self.subsitefolder, ssf)
        #------------------------
        # create mount point.
        if self.registerStorage(p):
            self.REQUEST.set('msg', 'register storage: %s' % self.REQUEST['err'])
            return None
        #------------------------
        # add plone site.
        if self.addSubSite(p):
            self.REQUEST.set('msg', 'addsubsite: %s' % self.REQUEST['err'])
            return None
        #------------------------
        # add apache virtual host.
        if SETUP_APACHE:
            if self.addAPAVH(p):
                self.REQUEST.set('msg', 'apache virtual host: %s' % self.REQUEST['err'])
                return None
        #------------------------
        # send welcome msg.
        self.sendto('welcome_template', 
                    {'e_mail': p.e_mail, 
                     'subject': 'Congratulation!', 
                     'domain_name': p.domain_name+'.'+p.base_name})
        self.sendto('welcome_template', 
                    {'e_mail': self.email_from_address, 
                     'subject': 'Congratulation!', 
                     'domain_name': p.domain_name+'.'+p.base_name})
        return 1

    security.declareProtected(ManagePortal, 'AllowPetition')
    def AllowPetition(self, id):
        ''' allow petition by id. '''
        p=self._getOb(id,None)
        if not p: return error
        if p.isAllowed(): return error
        result = self.createSubSite(p)
        if not result:
            return self.REQUEST.set('msg', '%s not created! %s' % (id, self.REQUEST['err']))
        p.setAllow()
        return None

    security.declareProtected(ManagePortal, 'AllowPetitions')
    def AllowPetitions(self, petition=[]):
        ''' allow petition by ids. '''
        results = 0
        for id in petition:
            result = self.AllowPetition(id)
            if not result:
                results += 1
        if not results:
            return self.REQUEST.RESPONSE.redirect(self.REQUEST['HTTP_REFERER']+'?portal_status_message=%s' % self.REQUEST['msg'])
        if results > 1:
            plural = 's'
        else:
            plural = ''
        return self.REQUEST.RESPONSE.redirect(self.REQUEST['HTTP_REFERER']+'?portal_status_message=Allow %s petition%s, %s' % (results, plural, self.REQUEST.get('msg', '')))

    def listScreenshots(self):
        ''' return the screenshots name list '''
        names=[]
        haveScreenshot = []
        for name, klass in custom_policies.items():
            available=getattr(klass, 'availableAtConstruction', None)
            if available and hasattr(klass, 'screenshot'):
                haveScreenshot.append(name)
        haveScreenshot.sort()
        for name in haveScreenshot:
            names.append({'name': name, 'screenshot': 1})
        names.append({'name': 'Default Plone(zh_tw)', 'screenshot': 0})
        return names

    def getScreenshot(self,name):
        ''' return the image object ''' 
        if custom_policies.has_key(name):
            klass = custom_policies[name]
            image = getattr(klass, 'screenshot', None)
            if image:
                screenshot = image['image']
                return screenshot.index_html(self.REQUEST, self.REQUEST.RESPONSE)
        return None

    def sendto( self, template, variables = {} ):
        ''' sends a confirm mail to someone '''
        if not variables: return

        variables['subject'] = '=?UTF-8?B?' + encodestring(variables['subject'])[:-1] + '?='

        send_from_name = '=?UTF-8?B?' + encodestring(self.email_from_name)[:-1] + '?='
        variables['send_from'] = r'%s <%s>' % (send_from_name, self.email_from_address)
        
        sendto_template = getattr(self, template, None)
        mail_text = sendto_template(self, **variables)
        mailhost = self.MailHost
        mailhost.send(mail_text,
            variables['e_mail'],
            variables['send_from'])

    #--------------------
    # Storages
    #--------------------
    security.declareProtected(ManagePortal, 'listStorages')
    def listStorages(self):
        ''' returns a list of storages 
            -> list of dicts with keys:(id,petitionId,timestamp)'''
        sids = [s.id for s in self.objectValues('Sub Site Storage')]
        
        res=[]
        
        for sid in sids:
            s=self._getOb(sid, None)
            if s:
                p = self._getOb(s.petitionId, None)
                if p:
                    url = 'http://' + '.'.join((p.domain_name, p.base_name))
                    title = p.title
                else: 
                    url = ''
                    title = ''
                res.append({'id':         sid, 
                            'petitionId': s.petitionId,
                            'url':        url,
                            'title':      title,
                            'timestamp':  s.Created(),
                            'assignPI':   getattr(s, 'assignPI', None)
                            })

        return res

    security.declareProtected('View', 'validStorages')
    def validStorages(self):
        ''' returns info of valid storages '''
        sids = [s.id for s in self.objectValues('Sub Site Storage') if s.isUsed()]
        res=[]
        for sid in sids:
            s=self._getOb(sid, None)
            if s:
                p = self._getOb(s.petitionId, None)
                if p:
                    url = 'http://' + '.'.join((p.domain_name, p.base_name))
                    company = p.company
                else: 
                    url = ''
                    company = ''
                res.append({'url':      url,
                            'company':  company,
                            'assignPI': getattr(s, 'assignPI', None)
                            })
        return res

    security.declareProtected(ManagePortal, 'nextStorage')
    def nextStorage(self):
        ''' returns a storage's instance for the next valid storage '''
        sids = [s.id for s in self.objectValues('Sub Site Storage') if s.isAvailable()]
        next = min(sids)
        s = self._getOb(next, None)
        return s

    security.declareProtected(ManagePortal, 'registerStorage')
    def registerStorage(self, p):
        ''' register petition id to a valid storage '''
        s = self.nextStorage()
        if s:
            s.regesterPI(p.id)
            p.storageId = s.id
            return None
        else:
            return error

    security.declareProtected(ManagePortal, 'createStorage')
    def createStorage(self, sid=""):
        ''' create a storage '''
        if not sid:
            sid = "%03d" % (len(self.objectIds('Sub Site Storage')) + 1)
        s = Storage(sid)
        self._setObject(sid, s)
        #--------------------
        # Add storage setting for zope
        zodbDIR = os.path.join(INSTANCE_HOME, 'var', 'subsite')
        settingDIR = os.path.join(INSTANCE_HOME, 'etc', 'subsite')
        if not os.access(zodbDIR, os.F_OK):
            os.makedirs(zodbDIR)
        if not os.access(settingDIR, os.F_OK):
            os.makedirs(settingDIR)
        # generate storage setting file
        portal = getToolByName(self, 'portal_url').getPortalObject()
        SSabsPath = '/'.join(getattr(portal, self.subsitefolder).getPhysicalPath())
        sf = open(os.path.join(settingDIR, sid), 'w')
        sf.write(ZS_SETTING % (sid, sid, DEF_QUOTA, SSabsPath, sid))
        sf.close()
        # write include setting to zope.conf
        includepath = "%%include %s" % ("%s/%s" % (settingDIR, sid))
        zopeconf = os.path.join(INSTANCE_HOME, 'etc', 'zope.conf')
        cmd = """echo "%s" >> %s """ % (includepath, zopeconf)
        os.system(cmd)
        s.includepath = includepath

    security.declareProtected(ManagePortal, 'createStorage')
    def createStorages(self, batch=5):
        ''' create storages '''
        while batch:
            self.createStorage()
            batch -= 1
        # try to restart zope
        if os.environ.has_key('ZMANAGED'):
            manage_restartable=1
            mtool = getToolByName(self, 'portal_membership')
            user = mtool.getAuthenticatedMember().getId()
            zLOG.LOG("SubSitePetitionTool", zLOG.INFO,
                     "Restart requested by %s" % user)
            #for db in Globals.opened: db.close()
            Lifetime.shutdown(1)
            return """<html>
            <head><meta HTTP-EQUIV=REFRESH CONTENT="10; URL=%s">
            </head>
            <body>Zope is restarting</body></html>
            """ % self.REQUEST['HTTP_REFERER']
        return self.REQUEST.RESPONSE.redirect(self.REQUEST['HTTP_REFERER'])

InitializeClass( SubSitePetitionTool )
